/*
ms_unipolar_drv v. d983.2019.03.09
Библиотека для низкокоуровневого управления 
бесколлекторными моторами (включая однополюсные и двухполюсные шаговые)
с использованием MotorShield для Arduino
() Copyleft GNU GPL v.3 dron983 and others
*/
#include <Arduino.h>
#include <ms_unipolar_drv.h>


// H# - признак изменения полярности выхода #
// Е# - включение выхода #
// Пины для управления Motor Shield
int H1, E1, H2, E2;




///*** Управление шаговым двигателем ***

// Массив последовательностей переключения выходов для одношагового режима
// Формат строки: (boolean H1State, boolean E1State, boolean H2State, boolean E2State)
boolean stChain[4][4] = {
 {0, 1, 0, 0},
 {0, 0, 0, 1},
 {1, 1, 0, 0},
 {0, 0, 1, 1} 
};

int stChainA[4][4] = {
 {0, 255, 0,   0},
 {0,   0, 0, 255},
 {1, 255, 0,   0},
 {0,   0, 1, 255} 
};

// Массив последовательностей переключения выходов для полушагового режима
// Формат строки: (int H1State, int E1Level, int H2State, int E2Level)
int stChainA2[8][4] = {
 {0, 255, 0,   0},
 {0, 127, 0, 127},
 {0,   0, 0, 255},
 {1, 127, 0, 127},
 {1, 255, 0,   0},
 {1, 127, 1, 127},
 {1,   0, 1, 255},
 {0, 127, 1, 127}
};


// Массив последовательностей переключения выходов для четвертьшагового режима
// Формат строки: (int H1State, int E1Level, int H2State, int E2Level)
int stChainA4[16][4] = {
 {0, 255, 0,   0},
 {0, 192, 0,  63},
 {0, 127, 0, 127},
 {0,  63, 0, 192},
 {0,   0, 0, 255},
 {1,  63, 0, 192},
 {1, 127, 0, 127},
 {1, 192, 0,  63},
 {1, 255, 0,   0},
 {1, 192, 1,  63},
 {1, 127, 1, 127},
 {1,  63, 1, 192},
 {1,   0, 1, 255},
 {0,  63, 1, 192},
 {0, 127, 1, 127},
 {0, 192, 1,  63}
};







     
TUniPolar::TUniPolar (int h1, int e1, int h2, int e2) {
  H1=h1;
  E1=e1;
  H2=h2;
  E2=e2;  
  pinMode(H1, OUTPUT);
  pinMode(E1, OUTPUT);
  pinMode(H2, OUTPUT);
  pinMode(E2, OUTPUT);
  anState = 0;
  anStep = 0;
  anCoord = 0;
}



// Эта функция непосредственно выставляет значение на пинах
// Шаг в цифровом одношаговом режиме
// Формат строки: (boolean H1State, boolean E1State, boolean H2State, boolean E2State)
void TUniPolar::doStepD(boolean stStr[4])
{
  digitalWrite(H1, stStr[0]);
  digitalWrite(E1, stStr[1]);
  digitalWrite(H2, stStr[2]);
  digitalWrite(E2, stStr[3]);

}

// Шаг в  ШИМ режиме
// Формат строки: (int H1State, int E1Level, int H2State, int E2Level)
void TUniPolar::doStepA (int stStr[4])
{
  digitalWrite(H1, stStr[0]);
  analogWrite(E1, stStr[1]);
  digitalWrite(H2, stStr[2]);
  analogWrite(E2, stStr[3]);

}


// Освобождение двигателя
void TUniPolar::doUnlock_()
{
  digitalWrite(H1, 0);
  digitalWrite(E1, 0);
  digitalWrite(H2, 0);
  digitalWrite(E2, 0);

}


// Останов двигателя со снижением тока в ШИМ режиме
// denum - коэффициент снижения тока (делитель)
// stArr - массив состояний выходов
void TUniPolar::uniStopA(int denum) {
  int  stState[4];
  stState[0]=stepArray[anState][0];
  stState[1]=trunc(stepArray[anState][1]/denum);
  stState[2]=stepArray[anState][2];
  stState[3]=trunc(stepArray[anState][3]/denum);
  doStepA (stState); 
}



// Вычисление номера текущего шага по состоянию портов в цифровом режиме
int TUniPolar::getStep() {
  int i,j;
  int st[4];
  boolean b;
  st[0]=digitalRead(H1);
  st[1]=digitalRead(E1);
  st[2]=digitalRead(H2);
  st[3]=digitalRead(E2); 
  for (i=0;i<=3;i++) {
    b=1;
    for (j=0;j<=3;j++) {
      b=b and (st[j]==stChain[i][j]);
    }
    if (b==1) {
      return i;
    }
  }
  return -1;
}


// Переключение обмоток для одного шага в цифровом одношаговом режиме в зависимости от текущего состояния
void TUniPolar::uniStep (bool dir) {
  int cState;
  cState=getStep();
  anStep++;   
   switch (dir){
      case 0 :
       cState++;
       if (cState > 3 ) cState=0;
       doStepD(stChain[cState]);    
       anCoord=anCoord+1;
       break;
      case 1:
       cState--;
       if (cState < 0 ) cState=3;
       doStepD(stChain[cState]);      
       anCoord=anCoord+1;
       break;     
   }

}

// Переключение обмоток для одного шага в ШИМ  режиме в зависимости от текущего состояния
// stArr - массив состояний выходов
// stCount - число строк в stArr
void TUniPolar::uniStepA (bool dir) {
   anStep=anStep+1/stepCount; 
   switch (dir){
      case 0 :
       anState++;
       if (anState > stepCount-1 ) anState=0;
       doStepA (stepArray[anState]);  
       anCoord=anCoord+1/stepCount;     
       break;
      case 1:
       anState--;
       if (anState < 0 ) anState=stepCount-1;
       doStepA (stepArray[anState]);        
       anCoord=anCoord-1/stepCount; 
       break;     
   } 
}

void TUniPolar::setModeA_1 () {   // Установка полношагового режима ШИМ
 stepArray=stChainA;
 stepCount=4;     
}

void TUniPolar::setModeA_2 () {   // Установка полушагового режима ШИМ
 stepArray=stChainA2;
 stepCount=8;     
}

void TUniPolar::setModeA_4 () {   // Установка четвертьшагового режима ШИМ
 stepArray=stChainA4;
 stepCount=16;     
}

// Установка произворльного режима ШИМ
// stArr - массив состояний выходов
// stCount - число строк в stArr
void TUniPolar::setModeA (int stArr[][4], int stCount) {
 stepArray=stArr;
 stepCount=stCount;   
}

float TUniPolar::readStep () {  // Чтение счётчика шагов
 return anStep; 
}

void TUniPolar::clearStep () { // Сброс счётчика шагов
  anStep = 0;
}

float TUniPolar::readCoord () {  // Чтение координаты (в шагах)
 return anCoord; 
}

void TUniPolar::clearCoord ()  { // Сброс координаты (в шагах)
  anCoord = 0;
}


void TUniPolar::uniUnlock() {
  doUnlock_();
}

// Получение текущего состояния обмоток в  режиме ШИМ
int* TUniPolar::getCurrStA() {
  return stepArray[anState];
}

// Получение следующего состояния обмоток в  режиме ШИМ
int* TUniPolar::getNextStA(bool dir) {
  int anNxState;
  switch (dir){
      case 0 :
       anNxState=anState+1;
       if (anNxState > stepCount-1 ) anNxState=0;
       return stepArray[anNxState];      
       break;
      case 1:
       anNxState=anState-1;
       if (anNxState < 0 ) anNxState=stepCount-1;
       return stepArray[anNxState];      
       break;     
   } 
  
}
  




